@@ -119,6 +119,7 @@ CTRL = "Meta" if sys.platform == 'darwin' else "Ctrl"
 # Fonts
 #==============================================================================
 def is_ubuntu():
+    "Detect if we are running in an Ubuntu-based distribution"
     if sys.platform.startswith('linux') and osp.isfile('/etc/lsb-release'):
         release_info = open('/etc/lsb-release').read()
         if 'Ubuntu' in release_info:
@@ -129,17 +130,34 @@ def is_ubuntu():
         return False
 
 
+def is_gtk_desktop():
+    "Detect if we are running in a Gtk-based desktop"
+    if sys.platform.startswith('linux'):
+        xdg_desktop = os.environ.get('XDG_CURRENT_DESKTOP', '')
+        if xdg_desktop:
+            gtk_desktops = ['Unity', 'GNOME', 'XFCE']
+            if any([xdg_desktop.startswith(d) for d in gtk_desktops]):
+                return True
+            else:
+                return False
+        else:
+            return False
+    else:
+        return False
+
+
 SANS_SERIF = ['Sans Serif', 'DejaVu Sans', 'Bitstream Vera Sans',
               'Bitstream Charter', 'Lucida Grande', 'MS Shell Dlg 2',
               'Calibri', 'Verdana', 'Geneva', 'Lucid', 'Arial',
               'Helvetica', 'Avant Garde', 'Times', 'sans-serif']
 
-MONOSPACE = ['Monospace', 'DejaVu Sans Mono', 'Consolas', 'Monaco',
+MONOSPACE = ['Monospace', 'DejaVu Sans Mono', 'Consolas',
              'Bitstream Vera Sans Mono', 'Andale Mono', 'Liberation Mono',
              'Courier New', 'Courier', 'monospace', 'Fixed', 'Terminal']
 
 
 if sys.platform == 'darwin':
+    MONOSPACE = ['Menlo'] + MONOSPACE
     BIG = MEDIUM = SMALL = 12
 elif os.name == 'nt':
     BIG = 12
@@ -710,7 +728,7 @@ DEFAULTS = [
 # 2. If you want to *remove* options that are no longer needed in our codebase,
 #    you need to do a MAJOR update in version, e.g. from 3.0.0 to 4.0.0
 # 3. You don't need to touch this value if you're just adding a new option
-CONF_VERSION = '15.1.0'
+CONF_VERSION = '15.2.0'
 
 # XXX: Previously we had load=(not DEV) here but DEV was set to *False*.
 # Check if it *really* needs to be updated or not
