@@ -6,6 +6,8 @@
 
 """Shell Interpreter"""
 
+from __future__ import print_function
+
 import sys
 import atexit
 import threading
@@ -20,6 +22,7 @@ from code import InteractiveConsole
 # Local imports:
 from spyderlib.utils.dochelpers import isdefined
 from spyderlib.utils import encoding
+from spyderlib.py3compat import is_text_string, getcwd
 from spyderlib.utils.misc import remove_backslashes
 
 # Force Python to search modules in the current directory first:
@@ -32,7 +35,7 @@ def guess_filename(filename):
         return filename
     if not filename.endswith('.py'):
         filename += '.py'
-    for path in [os.getcwdu()]+sys.path:
+    for path in [getcwd()] + sys.path:
         fname = osp.join(path, filename)
         if osp.isfile(fname):
             return fname
@@ -81,7 +84,7 @@ class Interpreter(InteractiveConsole, threading.Thread):
         # Create communication pipes
         pr, pw = os.pipe()
         self.stdin_read = os.fdopen(pr, "r")
-        self.stdin_write = os.fdopen(pw, "w", 0)
+        self.stdin_write = os.fdopen(pw, "wb", 0)
         self.stdout_write = Output()
         self.stderr_write = Output()
         
@@ -142,7 +145,7 @@ such as "spam", type "modules spam".
             try:
                 eval("pydoc.help(%s)" % text)
             except (NameError, SyntaxError):
-                print "no Python documentation found for '%r'" % text
+                print("no Python documentation found for '%r'" % text)
         self.write(os.linesep)
         self.widget_proxy.new_prompt("help> ")
         inp = self.raw_input_replacement()
@@ -210,8 +213,8 @@ has the same effect as typing a particular string at the help> prompt.
             # System ! command
             pipe = Popen(cmd[1:], shell=True,
                          stdin=PIPE, stderr=PIPE, stdout=PIPE)
-            txt_out = encoding.transcode( pipe.stdout.read() )
-            txt_err = encoding.transcode( pipe.stderr.read().rstrip() )
+            txt_out = encoding.transcode( pipe.stdout.read().decode() )
+            txt_err = encoding.transcode( pipe.stderr.read().decode().rstrip() )
             if txt_err:
                 self.stderr_write.write(txt_err)
             if txt_out:
@@ -245,7 +248,7 @@ has the same effect as typing a particular string at the help> prompt.
     def get_thread_id(self):
         """Return thread id"""
         if self._id is None:
-            for thread_id, obj in threading._active.items():
+            for thread_id, obj in list(threading._active.items()):
                 if obj is self:
                     self._id = thread_id
         return self._id
@@ -282,7 +285,7 @@ has the same effect as typing a particular string at the help> prompt.
         Run filename
         args: command line arguments (string)
         """
-        if args is not None and not isinstance(args, basestring):
+        if args is not None and not is_text_string(args):
             raise TypeError("expected a character buffer object")
         self.namespace['__file__'] = filename
         sys.argv = [filename]
@@ -299,7 +302,7 @@ has the same effect as typing a particular string at the help> prompt.
         where *obj* is the object represented by *text*
         and *valid* is True if object evaluation did not raise any exception
         """
-        assert isinstance(text, (str, unicode))
+        assert is_text_string(text)
         try:
             return eval(text, self.locals), True
         except:
