@@ -6,15 +6,12 @@
 
 """Configuration dialog / Preferences"""
 
-import os
 import os.path as osp
-import sys
 
-from spyderlib.baseconfig import _
-from spyderlib.config import CONF
+from spyderlib.baseconfig import _, running_in_mac_app
+from spyderlib.config import CONF, is_gtk_desktop
 from spyderlib.guiconfig import (CUSTOM_COLOR_SCHEME_NAME,
                                  set_default_color_scheme)
-from spyderlib.utils import programs
 from spyderlib.utils.qthelpers import get_icon, get_std_icon
 from spyderlib.userconfig import NoDefault
 from spyderlib.widgets.colors import ColorLayout
@@ -27,7 +24,7 @@ from spyderlib.qt.QtGui import (QWidget, QDialog, QListWidget, QListWidgetItem,
                                 QPushButton, QFontComboBox, QGroupBox,
                                 QComboBox, QColor, QGridLayout, QTabWidget,
                                 QRadioButton, QButtonGroup, QSplitter,
-                                QStyleFactory, QScrollArea)
+                                QStyleFactory, QScrollArea, QDoubleSpinBox)
 from spyderlib.qt.QtCore import Qt, QSize, SIGNAL, SLOT, Slot
 from spyderlib.qt.compat import (to_qvariant, from_qvariant,
                                  getexistingdirectory, getopenfilename)
@@ -278,8 +275,12 @@ class SpyderConfigPage(ConfigPage, ConfigAccessMixin):
                          lambda _foo, opt=option: self.has_been_modified(opt))
         for spinbox, (option, default) in list(self.spinboxes.items()):
             spinbox.setValue(self.get_option(option, default))
-            self.connect(spinbox, SIGNAL('valueChanged(int)'),
-                         lambda _foo, opt=option: self.has_been_modified(opt))
+            if type(spinbox) is QSpinBox:
+                self.connect(spinbox, SIGNAL('valueChanged(int)'),
+                             lambda _foo, opt=option: self.has_been_modified(opt))
+            else:
+                self.connect(spinbox, SIGNAL('valueChanged(double)'),
+                             lambda _foo, opt=option: self.has_been_modified(opt))
         for combobox, (option, default) in list(self.comboboxes.items()):
             value = self.get_option(option, default)
             for index in range(combobox.count()):
@@ -492,13 +493,19 @@ class SpyderConfigPage(ConfigPage, ConfigAccessMixin):
             slabel = QLabel(suffix)
         else:
             slabel = None
-        spinbox = QSpinBox()
+        if step is not None:
+            if type(step) is int:
+                spinbox = QSpinBox()
+            else:
+                spinbox = QDoubleSpinBox()
+                spinbox.setDecimals(1)
+            spinbox.setSingleStep(step)
+        else:
+            spinbox = QSpinBox()
         if min_ is not None:
             spinbox.setMinimum(min_)
         if max_ is not None:
             spinbox.setMaximum(max_)
-        if step is not None:
-            spinbox.setSingleStep(step)
         if tip is not None:
             spinbox.setToolTip(tip)
         self.spinboxes[spinbox] = (option, default)
@@ -656,6 +663,11 @@ class MainConfigPage(GeneralConfigPage):
         # --- Interface
         interface_group = QGroupBox(_("Interface"))
         styles = [str(txt) for txt in list(QStyleFactory.keys())]
+        # Don't offer users the possibility to change to a different
+        # style in Gtk-based desktops
+        # Fixes Issue 2036
+        if is_gtk_desktop() and ('GTK+' in styles):
+            styles = ['GTK+']
         choices = list(zip(styles, [style.lower() for style in styles]))
         style_combo = self.create_combobox(_('Qt windows style'), choices,
                                            'windows_style',
@@ -687,7 +699,7 @@ class MainConfigPage(GeneralConfigPage):
         margins_layout.addWidget(margin_spin)
 
         # Decide if it's possible to activate or not singie instance mode
-        if sys.platform == "darwin" and 'Spyder.app' in __file__:
+        if running_in_mac_app():
             self.set_option("single_instance", True)
             single_instance_box.setEnabled(False)
         
