@@ -29,10 +29,16 @@ import shutil
 
 # Local imports
 from spyderlib.utils.qthelpers import (get_icon, create_action, add_actions,
-                                       file_uri)
+                                       file_uri, get_std_icon)
 from spyderlib.utils import misc, encoding, programs, vcs
 from spyderlib.baseconfig import _
-from spyderlib.py3compat import to_text_string, getcwd, str_lower
+from spyderlib.py3compat import (to_text_string, to_binary_string, getcwd,
+                                 str_lower)
+
+try:
+    from IPython.nbconvert import PythonExporter as nbexporter
+except:
+    nbexporter = None      # analysis:ignore
 
 
 def fixpath(path):
@@ -81,7 +87,6 @@ class DirView(QTreeView):
         super(DirView, self).__init__(parent)
         self.name_filters = None
         self.parent_widget = parent
-        self.valid_types = None
         self.show_all = None
         self.menu = None
         self.common_actions = None
@@ -150,13 +155,11 @@ class DirView(QTreeView):
                 return osp.dirname(fname)
         
     #---- Tree view widget
-    def setup(self, name_filters=['*.py', '*.pyw'],
-              valid_types= ('.py', '.pyw'), show_all=False):
+    def setup(self, name_filters=['*.py', '*.pyw'], show_all=False):
         """Setup tree widget"""
         self.setup_view()
         
         self.set_name_filters(name_filters)
-        self.valid_types = valid_types
         self.show_all = show_all
         
         # Setup context menu
@@ -225,8 +228,9 @@ class DirView(QTreeView):
         only_files = all([osp.isfile(_fn) for _fn in fnames])
         only_modules = all([osp.splitext(_fn)[1] in ('.py', '.pyw', '.ipy')
                             for _fn in fnames])
-        only_valid = all([osp.splitext(_fn)[1] in self.valid_types
-                          for _fn in fnames])
+        only_notebooks = all([osp.splitext(_fn)[1] == '.ipynb'
+                              for _fn in fnames])
+        only_valid = all([encoding.is_text_file(_fn) for _fn in fnames])
         run_action = create_action(self, _("Run"), icon="run_small.png",
                                    triggered=self.run)
         edit_action = create_action(self, _("Edit"), icon="edit.png",
@@ -241,6 +245,9 @@ class DirView(QTreeView):
                                       icon="rename.png",
                                       triggered=self.rename)
         open_action = create_action(self, _("Open"), triggered=self.open)
+        ipynb_convert_action = create_action(self, _("Convert to Python script"),
+                                             icon="python.png",
+                                             triggered=self.convert)
         
         actions = []
         if only_modules:
@@ -254,6 +261,8 @@ class DirView(QTreeView):
         if all([fixpath(osp.dirname(_fn)) == basedir for _fn in fnames]):
             actions.append(move_action)
         actions += [None]
+        if only_notebooks and nbexporter is not None:
+            actions.append(ipynb_convert_action)
         
         # VCS support is quite limited for now, so we are enabling the VCS
         # related actions only when a single file/folder is selected:
@@ -282,7 +291,7 @@ class DirView(QTreeView):
                                triggered=lambda fnames=fnames:
                                self.open_terminal(fnames))
         actions.append(action)
-        _title = _("Open Python interpreter here")
+        _title = _("Open Python console here")
         action = create_action(self, _title, icon="python.png",
                                triggered=lambda fnames=fnames:
                                self.open_interpreter(fnames))
@@ -405,8 +414,7 @@ class DirView(QTreeView):
         if fnames is None:
             fnames = self.get_selected_filenames()
         for fname in fnames:
-            ext = osp.splitext(fname)[1]
-            if osp.isfile(fname) and ext in self.valid_types:
+            if osp.isfile(fname) and encoding.is_text_file(fname):
                 self.parent_widget.sig_open_file.emit(fname)
             else:
                 self.open_outside_spyder([fname])
@@ -490,6 +498,27 @@ class DirView(QTreeView):
                 # Canceled
                 return
 
+    def convert_notebook(self, fname):
+        """Convert an IPython notebook to a Python script in editor"""
+        try: 
+            script = nbexporter().from_filename(fname)[0]
+        except Exception as e:
+            QMessageBox.critical(self, _('Conversion error'), 
+                                 _("It was not possible to convert this "
+                                 "notebook. The error is:\n\n") + \
+                                 to_text_string(e))
+            return
+        self.parent_widget.sig_new_file.emit(script)
+    
+    def convert(self, fnames=None):
+        """Convert IPython notebooks to Python scripts in editor"""
+        if fnames is None:
+            fnames = self.get_selected_filenames()
+        if not isinstance(fnames, (tuple, list)):
+            fnames = [fnames]
+        for fname in fnames:
+            self.convert_notebook(fname)
+
     def rename_file(self, fname):
         """Rename file"""
         path, valid = QInputDialog.getText(self, _('Rename'),
@@ -573,7 +602,8 @@ class DirView(QTreeView):
                 if is_package:
                     fname = osp.join(dirname, '__init__.py')
                     try:
-                        open(fname, 'wb').write('#')
+                        with open(fname, 'wb') as f:
+                            f.write(to_binary_string('#'))
                         return dirname
                     except EnvironmentError as error:
                         QMessageBox.critical(self, title,
@@ -624,7 +654,8 @@ class DirView(QTreeView):
             if osp.splitext(fname)[1] in ('.py', '.pyw', '.ipy'):
                 create_script(fname)
             else:
-                open(fname, 'wb').write('')
+                with open(fname, 'wb') as f:
+                    f.write(to_binary_string(''))
         fname = self.create_new_file(basedir, title, filters, create_func)
         if fname is not None:
             self.open([fname])
@@ -944,30 +975,26 @@ class ExplorerWidget(QWidget):
     """Explorer widget"""
     sig_option_changed = Signal(str, object)
     sig_open_file = Signal(str)
+    sig_new_file = Signal(str)
     
     def __init__(self, parent=None, name_filters=['*.py', '*.pyw'],
-                 valid_types=('.py', '.pyw'), show_all=False,
-                 show_cd_only=None, show_toolbar=True, show_icontext=True):
+                 show_all=False, show_cd_only=None, show_icontext=True):
         QWidget.__init__(self, parent)
         
         self.treewidget = ExplorerTreeWidget(self, show_cd_only=show_cd_only)
-        self.treewidget.setup(name_filters=name_filters,
-                              valid_types=valid_types, show_all=show_all)
+        self.treewidget.setup(name_filters=name_filters, show_all=show_all)
         self.treewidget.chdir(getcwd())
         
-        toolbar_action = create_action(self, _("Show toolbar"),
-                                       toggled=self.toggle_toolbar)
         icontext_action = create_action(self, _("Show icons and text"),
                                         toggled=self.toggle_icontext)
-        self.treewidget.common_actions += [None,
-                                           toolbar_action, icontext_action]
+        self.treewidget.common_actions += [None, icontext_action]
         
         # Setup toolbar
         self.toolbar = QToolBar(self)
         self.toolbar.setIconSize(QSize(16, 16))
         
         self.previous_action = create_action(self, text=_("Previous"),
-                            icon=get_icon('previous.png'),
+                            icon=get_std_icon("ArrowBack"),
                             triggered=self.treewidget.go_to_previous_directory)
         self.toolbar.addAction(self.previous_action)
         self.previous_action.setEnabled(False)
@@ -975,7 +1002,7 @@ class ExplorerWidget(QWidget):
                      self.previous_action.setEnabled)
         
         self.next_action = create_action(self, text=_("Next"),
-                            icon=get_icon('next.png'),
+                            icon=get_std_icon("ArrowForward"),
                             triggered=self.treewidget.go_to_next_directory)
         self.toolbar.addAction(self.next_action)
         self.next_action.setEnabled(False)
@@ -983,9 +1010,10 @@ class ExplorerWidget(QWidget):
                      self.next_action.setEnabled)
         
         parent_action = create_action(self, text=_("Parent"),
-                            icon=get_icon('up.png'),
+                            icon=get_std_icon("ArrowUp"),
                             triggered=self.treewidget.go_to_parent_directory)
         self.toolbar.addAction(parent_action)
+        self.toolbar.addSeparator()
 
         options_action = create_action(self, text='', tip=_("Options"),
                                        icon=get_icon('tooloptions.png'))
@@ -996,8 +1024,6 @@ class ExplorerWidget(QWidget):
         add_actions(menu, self.treewidget.common_actions)
         options_action.setMenu(menu)
             
-        toolbar_action.setChecked(show_toolbar)
-        self.toggle_toolbar(show_toolbar)   
         icontext_action.setChecked(show_icontext)
         self.toggle_icontext(show_icontext)     
         
@@ -1005,21 +1031,17 @@ class ExplorerWidget(QWidget):
         vlayout.addWidget(self.toolbar)
         vlayout.addWidget(self.treewidget)
         self.setLayout(vlayout)
-        
-    def toggle_toolbar(self, state):
-        """Toggle toolbar"""
-        self.sig_option_changed.emit('show_toolbar', state)
-        self.toolbar.setVisible(state)
-            
+
     def toggle_icontext(self, state):
         """Toggle icon text"""
         self.sig_option_changed.emit('show_icontext', state)
         for action in self.toolbar.actions():
-            widget = self.toolbar.widgetForAction(action)
-            if state:
-                widget.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-            else:
-                widget.setToolButtonStyle(Qt.ToolButtonIconOnly)
+            if not action.isSeparator():
+                widget = self.toolbar.widgetForAction(action)
+                if state:
+                    widget.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+                else:
+                    widget.setToolButtonStyle(Qt.ToolButtonIconOnly)
 
 
 class FileExplorerTest(QWidget):
@@ -1082,4 +1104,4 @@ if __name__ == "__main__":
     test.resize(640, 480)
     test.show()
     sys.exit(app.exec_())
-    
\ No newline at end of file
+    
