@@ -1,5 +1,9 @@
-# -*- coding: utf-8 -*-
+#
+# IMPORTANT NOTE: Don't add a coding line here! It's not necessary for
+# site files
+#
 # Spyder's ExternalPythonShell sitecustomize
+#
 
 import sys
 import os
@@ -8,12 +12,18 @@ import pdb
 import bdb
 
 
+PY2 = sys.version[0] == '2'
+
+
+#==============================================================================
 # sys.argv can be missing when Python is embedded, taking care of it.
 # Fixes Issue 1473 and other crazy crashes with IPython 0.13 trying to
 # access it.
+#==============================================================================
 if not hasattr(sys, 'argv'):
     sys.argv = ['']
 
+
 #==============================================================================
 # Important Note:
 #
@@ -37,8 +47,38 @@ def _print(*objects, **options):
         else:
             print >>file, string,
 
+
+#==============================================================================
+# Execfile functions
+# 
+# The definitions for Python 2 on Windows were taken from the IPython
+# project (present in IPython.utils.py3compat)
+# Copyright (C) The IPython Development Team
+# Distributed under the terms of the modified BSD license
+#==============================================================================
 try:
+    # Python 2
     import __builtin__ as builtins
+    if os.name == 'nt':
+        def encode(u):
+            return u.encode('utf8', 'replace')
+        def execfile(fname, glob=None, loc=None):
+            loc = loc if (loc is not None) else glob
+            scripttext = builtins.open(fname).read()+ '\n'
+            # compile converts unicode filename to str assuming
+            # ascii. Let's do the conversion before calling compile
+            if isinstance(fname, unicode):
+                filename = encode(fname)
+            else:
+                filename = fname
+            exec(compile(scripttext, filename, 'exec'), glob, loc)
+    else:
+        def execfile(fname, *where):
+            if isinstance(fname, unicode):
+                filename = fname.encode(sys.getfilesystemencoding())
+            else:
+                filename = fname
+            builtins.execfile(filename, *where)
 except ImportError:
     # Python 3
     import builtins
@@ -47,7 +87,10 @@ except ImportError:
         # Open a source file correctly, whatever its encoding is
         exec(compile(open(filename, 'rb').read(), filename, 'exec'), namespace)
 
-# Colorization of sys.stderr (standard Python interpreter)
+
+#==============================================================================
+# Colorization of sys.stderr (standard Python interpreter) 
+#==============================================================================
 if os.environ.get("COLORIZE_SYS_STDERR", "").lower() == "true":
     class StderrProxy(object):
         """Proxy to sys.stderr file object overriding only the `write` method 
@@ -79,8 +122,10 @@ if os.environ.get("COLORIZE_SYS_STDERR", "").lower() == "true":
     stderrproxy = StderrProxy()
 
 
+#==============================================================================
 # Prepending this spyderlib package's path to sys.path to be sure 
 # that another version of spyderlib won't be imported instead:
+#==============================================================================
 spyderlib_path = osp.dirname(__file__)
 while not osp.isdir(osp.join(spyderlib_path, 'spyderlib')):
     spyderlib_path = osp.abspath(osp.join(spyderlib_path, os.pardir))
@@ -94,7 +139,9 @@ if not spyderlib_path.startswith(sys.prefix):
 os.environ['SPYDER_PARENT_DIR'] = spyderlib_path
 
 
+#==============================================================================
 # Set PyQt4 API to #1 or #2
+#==============================================================================
 pyqt_api = int(os.environ.get("PYQT_API", "0"))
 if pyqt_api:
     try:
@@ -109,9 +156,11 @@ if pyqt_api:
         pass
 
 
-if os.name == 'nt': # Windows platforms
-            
-    # Setting console encoding (otherwise Python does not recognize encoding)
+#==============================================================================
+# Setting console encoding (otherwise Python does not recognize encoding)
+# for Windows platforms
+#==============================================================================
+if os.name == 'nt':
     try:
         import locale, ctypes
         _t, _cp = locale.getdefaultlocale('LANG')
@@ -126,87 +175,67 @@ if os.name == 'nt': # Windows platforms
         pass
 
 
-# For our MacOs X app
-if sys.platform == 'darwin' and 'Spyder.app' in __file__:
-    interpreter = os.environ.get('SPYDER_INTERPRETER')
-    if 'Spyder.app' not in interpreter:
-        # We added this file's dir to PYTHONPATH (in pythonshell.py)
-        # so that external interpreters can import this script, and
-        # now we are removing it
-        del os.environ['PYTHONPATH']
-
-        # Add a minimal library (with spyderlib) at the end of sys.path to
-        # be able to connect our monitor to the external console
-        app_pythonpath = 'Spyder.app/Contents/Resources/lib/python2.7'
-        full_pythonpath = [p for p in sys.path if p.endswith(app_pythonpath)]
-        if full_pythonpath:
-            sys.path.remove(full_pythonpath[0])
-            sys.path.append(full_pythonpath[0] + osp.sep + 'minimal-lib')
-    else:
-        # Add missing variables and methods to the app's site module
-        import site
-        import osx_app_site
-        osx_app_site.setcopyright()
-        osx_app_site.sethelper()
-        site._Printer = osx_app_site._Printer
-        site.USER_BASE = osx_app_site.getuserbase()
-        site.USER_SITE = osx_app_site.getusersitepackages()
+#==============================================================================
+# Settings for our MacOs X app
+#==============================================================================
+if sys.platform == 'darwin':
+    from spyderlib.baseconfig import MAC_APP_NAME
+    if MAC_APP_NAME in __file__:
+        interpreter = os.environ.get('SPYDER_INTERPRETER')
+        if MAC_APP_NAME not in interpreter:
+            # Add a minimal library (with spyderlib) at the end of sys.path to
+            # be able to connect our monitor to the external console
+            py_ver = '%s.%s' % (sys.version_info[0], sys.version_info[1])
+            app_pythonpath = '%s/Contents/Resources/lib/python%s' % (MAC_APP_NAME,
+                                                                     py_ver)
+            full_pythonpath = [p for p in sys.path if p.endswith(app_pythonpath)]
+            if full_pythonpath:
+                sys.path.remove(full_pythonpath[0])
+                sys.path.append(full_pythonpath[0] + osp.sep + 'minimal-lib')
+        else:
+            # Add missing variables and methods to the app's site module
+            import site
+            import osx_app_site
+            osx_app_site.setcopyright()
+            osx_app_site.sethelper()
+            site._Printer = osx_app_site._Printer
+            site.USER_BASE = osx_app_site.getuserbase()
+            site.USER_SITE = osx_app_site.getusersitepackages()
 
 
-mpl_backend = os.environ.get("MATPLOTLIB_BACKEND")
-if mpl_backend:
-    try:
-        import matplotlib
-        if os.environ.get('QT_API') == 'pyside':
-            # Try to address PySide lack of an input hook on Mac by settting
-            # mpl_backend to always be MacOSX
-            # Fixes Issue 347
-            if mpl_backend == 'Qt4Agg' and sys.platform == 'darwin':
-                mpl_backend = 'MacOSX'
-        matplotlib.rcParams['docstring.hardcopy'] = True
-        matplotlib.rcParams['interactive'] = True
-        matplotlib.use(mpl_backend)
-    except ImportError:
-        pass
-
-
-if os.environ.get("MATPLOTLIB_PATCH", "").lower() == "true":
-    try:
-        from spyderlib import mpl_patch
-        mpl_patch.apply()
-    except ImportError:
-        pass
-
-
-# Set standard outputs encoding:
-# (otherwise, for example, print("é") will fail)
-encoding = None
-try:
-    import locale
-except ImportError:
-    pass
-else:
-    loc = locale.getdefaultlocale()
-    if loc[1]:
-        encoding = loc[1]
-
-if encoding is None:
-    encoding = "UTF-8"
-
-try:
-    sys.setdefaultencoding(encoding)
-    os.environ['SPYDER_ENCODING'] = encoding
-except AttributeError:
-    # Python 3
-    pass
-    
+#==============================================================================
+# Importing user's sitecustomize
+#==============================================================================
 try:
     import sitecustomize  #analysis:ignore
 except ImportError:
     pass
 
 
+#==============================================================================
+# Add default filesystem encoding on Linux to avoid an error with
+# Matplotlib 1.5 in Python 2 (Fixes Issue 2793) 
+#==============================================================================
+if PY2 and sys.platform.startswith('linux'):
+    def _getfilesystemencoding_wrapper():
+        return 'utf-8'
+
+    sys.getfilesystemencoding = _getfilesystemencoding_wrapper
+
+
+#==============================================================================
+# Importing matplotlib before creating the monitor.
+# This prevents a kernel crash with the inline backend in our IPython
+# consoles on Linux and Python 3 (Fixes Issue 2257)
+#==============================================================================
+try:
+    import matplotlib
+except ImportError:
+    matplotlib = None   # analysis:ignore
+
+#==============================================================================
 # Communication between Spyder and the remote process
+#==============================================================================
 if os.environ.get('SPYDER_SHELL_ID') is None:
     monitor = None
 else:
@@ -245,95 +274,114 @@ else:
             _print("Can't open file %s" % source, file=sys.stderr)
     builtins.open_in_spyder = open_in_spyder
     
-    # * PyQt4:
-    #   * Removing PyQt4 input hook which is not working well on Windows since 
-    #     opening a subprocess do not attach a real console to it
-    #     (with keyboard events...)
-    #   * Replacing it with our own input hook
-    # * PySide:
-    #   * Installing an input hook: this feature is not yet supported 
-    #     natively by PySide
-    if os.environ.get("INSTALL_QT_INPUTHOOK", "").lower() == "true":
-        if os.environ["QT_API"] == 'pyqt':
-            from PyQt4 import QtCore
-            # Removing PyQt's PyOS_InputHook implementation:
-            QtCore.pyqtRemoveInputHook()
-        elif os.environ["QT_API"] == 'pyside':
-            from PySide import QtCore
-            # XXX: when PySide will implement an input hook, we will have to 
-            # remove it here
-        else:
-            assert False
+    if os.environ["QT_API"] == 'pyqt':
+        from PyQt4 import QtCore
+    elif os.environ["QT_API"] == 'pyside':
+        from PySide import QtCore          #analysis:ignore
 
-        def qt_inputhook():
-            """Qt input hook for Spyder's console
-            
-            This input hook wait for available stdin data (notified by
-            ExternalPythonShell through the monitor's inputhook_flag
-            attribute), and in the meantime it processes Qt events."""
-            # Refreshing variable explorer, except on first input hook call:
-            # (otherwise, on slow machines, this may freeze Spyder)
-            monitor.refresh_from_inputhook()
-            if os.name == 'nt':
-                try:
-                    # This call fails for Python without readline support
-                    # (or on Windows platforms) when PyOS_InputHook is called
-                    # for the second consecutive time, because the 100-bytes
-                    # stdin buffer is full.
-                    # For more details, see the `PyOS_StdioReadline` function
-                    # in Python source code (Parser/myreadline.c)
-                    sys.stdin.tell()
-                except IOError:
-                    return 0
-            app = QtCore.QCoreApplication.instance()
-            if app and app.thread() is QtCore.QThread.currentThread():
-                timer = QtCore.QTimer()
-                QtCore.QObject.connect(timer, QtCore.SIGNAL('timeout()'),
-                                       app, QtCore.SLOT('quit()'))
-                monitor.toggle_inputhook_flag(False)
-                while not monitor.inputhook_flag:
-                    timer.start(50)
-                    QtCore.QCoreApplication.exec_()
-                    timer.stop()
-#                # Socket-based alternative:
-#                socket = QtNetwork.QLocalSocket()
-#                socket.connectToServer(os.environ['SPYDER_SHELL_ID'])
-#                socket.waitForConnected(-1)
-#                while not socket.waitForReadyRead(10):
-#                    timer.start(50)
-#                    QtCore.QCoreApplication.exec_()
-#                    timer.stop()
-#                socket.read(3)
-#                socket.disconnectFromServer()
-            return 0
-
-        # Installing Spyder's PyOS_InputHook implementation:
-        import ctypes
-        cb_pyfunctype = ctypes.PYFUNCTYPE(ctypes.c_int)(qt_inputhook)
-        pyos_ih = ctypes.c_void_p.in_dll(ctypes.pythonapi, "PyOS_InputHook")
-        pyos_ih.value = ctypes.cast(cb_pyfunctype, ctypes.c_void_p).value
-    else:
-        # Quite limited feature: notify only when a result is displayed in
-        # console (does not notify at every prompt)
-        def displayhook(obj):
-            sys.__displayhook__(obj)
-            monitor.refresh()
+    def qt_nt_inputhook():
+        """Qt input hook for Windows
+        
+        This input hook wait for available stdin data (notified by
+        ExternalPythonShell through the monitor's inputhook_flag
+        attribute), and in the meantime it processes Qt events.
+        """
+        # Refreshing variable explorer, except on first input hook call:
+        # (otherwise, on slow machines, this may freeze Spyder)
+        monitor.refresh_from_inputhook()
+        if os.name == 'nt':
+            try:
+                # This call fails for Python without readline support
+                # (or on Windows platforms) when PyOS_InputHook is called
+                # for the second consecutive time, because the 100-bytes
+                # stdin buffer is full.
+                # For more details, see the `PyOS_StdioReadline` function
+                # in Python source code (Parser/myreadline.c)
+                sys.stdin.tell()
+            except IOError:
+                return 0
+        app = QtCore.QCoreApplication.instance()
+        if app and app.thread() is QtCore.QThread.currentThread():
+            timer = QtCore.QTimer()
+            QtCore.QObject.connect(timer, QtCore.SIGNAL('timeout()'),
+                                   app, QtCore.SLOT('quit()'))
+            monitor.toggle_inputhook_flag(False)
+            while not monitor.inputhook_flag:
+                timer.start(50)
+                QtCore.QCoreApplication.exec_()
+                timer.stop()
+#            # Socket-based alternative:
+#            socket = QtNetwork.QLocalSocket()
+#            socket.connectToServer(os.environ['SPYDER_SHELL_ID'])
+#            socket.waitForConnected(-1)
+#            while not socket.waitForReadyRead(10):
+#                timer.start(50)
+#                QtCore.QCoreApplication.exec_()
+#                timer.stop()
+#            socket.read(3)
+#            socket.disconnectFromServer()
+        return 0
     
-        sys.displayhook = displayhook
+
+#==============================================================================
+# Matplotlib settings
+#==============================================================================
+if matplotlib is not None:
+    mpl_backend = os.environ.get("MATPLOTLIB_BACKEND", "")
+    mpl_ion = os.environ.get("MATPLOTLIB_ION", "")
+    if not mpl_backend:
+        mpl_backend = 'Qt4Agg'
+
+    # To have mpl docstrings as rst
+    matplotlib.rcParams['docstring.hardcopy'] = True
+
+    # Activate interactive mode when needed
+    if mpl_ion.lower() == "true":
+        matplotlib.rcParams['interactive'] = True
+
+    if os.environ.get("IPYTHON_KERNEL", "").lower() != "true":
+        import ctypes
+        from spyderlib.widgets.externalshell import inputhooks
+
+        # Setting the user defined backend
+        matplotlib.use(mpl_backend)
+
+        # Setting the right input hook according to mpl_backend,
+        # IMPORTANT NOTE: Don't try to abstract the steps to set a PyOS
+        # input hook callback in a function. It will *crash* the
+        # interpreter!!
+        if mpl_backend == "Qt4Agg" and os.name == 'nt' and \
+          monitor is not None:
+            # Removing PyQt4 input hook which is not working well on
+            # Windows since opening a subprocess does not attach a real
+            # console to it (with keyboard events...)
+            if os.environ["QT_API"] == 'pyqt': 
+                inputhooks.remove_pyqt_inputhook()
+            # Using our own input hook
+            # NOTE: it's not working correctly for some configurations
+            # (See issue 1831)
+            callback = inputhooks.set_pyft_callback(qt_nt_inputhook)
+            pyos_ih = inputhooks.get_pyos_inputhook()
+            pyos_ih.value = ctypes.cast(callback, ctypes.c_void_p).value
+        elif mpl_backend == "Qt4Agg" and os.environ["QT_API"] == 'pyside':
+            # PySide doesn't have an input hook, so we need to install one
+            # to be able to show plots.
+            # Note: This only works well for Posix systems
+            callback = inputhooks.set_pyft_callback(inputhooks.qt4)
+            pyos_ih = inputhooks.get_pyos_inputhook()
+            pyos_ih.value = ctypes.cast(callback, ctypes.c_void_p).value
+        elif mpl_backend != "Qt4Agg" and os.environ["QT_API"] == 'pyqt':
+            # Matplotlib backends install their own input hooks, so we
+            # need to remove the PyQt one to make them work
+            inputhooks.remove_pyqt_inputhook()
 
 
-#===============================================================================
-# Monkey-patching pdb
-#===============================================================================
-
+#==============================================================================
+# IPython adjustments
+#==============================================================================
 if os.environ.get("IPYTHON_KERNEL", "").lower() == "true":
 
-    #XXX If Matplotlib is not imported first, the next IPython import will fail
-    try:
-        import matplotlib  # analysis:ignore
-    except ImportError:
-        pass
-
+    # Use ipydb as the debugger to patch on IPython consoles
     from IPython.core.debugger import Pdb as ipyPdb
     pdb.Pdb = ipyPdb
 
@@ -348,9 +396,26 @@ if os.environ.get("IPYTHON_KERNEL", "").lower() == "true":
             kwargs['testRunner'] = kwargs.pop('testRunner', test_runner)
             kwargs['exit'] = False
             TestProgram.__init__(self, *args, **kwargs)
-
     unittest.main = IPyTesProgram
+    
+    # Pandas monkey-patches
+    try:
+        # Make Pandas recognize our IPython consoles as proper qtconsoles
+        # Fixes Issue 2015
+        def in_qtconsole():
+            return True
+        import pandas as pd
+        pd.core.common.in_qtconsole = in_qtconsole
+        
+        # Set Pandas output encoding
+        pd.options.display.encoding = 'utf-8'
+    except (ImportError, AttributeError):
+        pass
+        
 
+#==============================================================================
+# Pdb adjustments
+#==============================================================================
 class SpyderPdb(pdb.Pdb):
     def set_spyder_breakpoints(self):
         self.clear_all_breaks()
@@ -377,6 +442,11 @@ class SpyderPdb(pdb.Pdb):
         if not frame:
             return
         fname = self.canonic(frame.f_code.co_filename)
+        if sys.version[0] == '2':
+            try:
+                fname = unicode(fname, "utf-8")
+            except TypeError:
+                pass
         lineno = frame.f_lineno
         if isinstance(fname, basestring) and isinstance(lineno, int):
             if osp.isfile(fname) and monitor is not None:
@@ -451,19 +521,55 @@ def postcmd(self, stop, line):
     self.notify_spyder(self.curframe)
     return self._old_Pdb_postcmd(stop, line)
 
+# Breakpoints don't work for files with non-ascii chars in Python 2
+# Fixes Issue 1484
+if sys.version[0] == '2':
+    @monkeypatch_method(pdb.Pdb, 'Pdb')
+    def break_here(self, frame):
+        from bdb import effective
+        filename = self.canonic(frame.f_code.co_filename)
+        try:
+            filename = unicode(filename, "utf-8")
+        except TypeError:
+            pass
+        if not filename in self.breaks:
+            return False
+        lineno = frame.f_lineno
+        if not lineno in self.breaks[filename]:
+            # The line itself has no breakpoint, but maybe the line is the
+            # first line of a function with breakpoint set by function name.
+            lineno = frame.f_code.co_firstlineno
+            if not lineno in self.breaks[filename]:
+                return False
+    
+        # flag says ok to delete temp. bp
+        (bp, flag) = effective(filename, lineno, frame)
+        if bp:
+            self.currentbp = bp.number
+            if (flag and bp.temporary):
+                self.do_clear(str(bp.number))
+            return True
+        else:
+            return False
 
-# Restoring (almost) original sys.path:
+
+#==============================================================================
+# # Restoring (almost) original sys.path:
 # (Note: do not remove spyderlib_path from sys.path because if Spyder has been
 #  installed using python setup.py install, then this could remove the 
 #  'site-packages' directory from sys.path!)
+#==============================================================================
 try:
     sys.path.remove(osp.join(spyderlib_path,
                              "spyderlib", "widgets", "externalshell"))
 except ValueError:
     pass
 
+
+#==============================================================================
 # Ignore PyQt4's sip API changes (this should be used wisely -e.g. for
-# debugging- as dynamic API change is not supported by PyQt)
+# debugging- as dynamic API change is not supported by PyQt) 
+#==============================================================================
 if os.environ.get("IGNORE_SIP_SETAPI_ERRORS", "").lower() == "true":
     try:
         import sip
@@ -478,11 +584,12 @@ if os.environ.get("IGNORE_SIP_SETAPI_ERRORS", "").lower() == "true":
         pass
 
 
-# The following classes and functions are mainly intended to be used from 
-# an interactive Python session
-class UserModuleDeleter(object):
+#==============================================================================
+# User module reloader
+#==============================================================================
+class UserModuleReloader(object):
     """
-    User Module Deleter (UMD) aims at deleting user modules 
+    User Module Reloader (UMR) aims at deleting user modules 
     to force Python to deeply reload them during import
     
     pathlist [list]: blacklist in terms of module path
@@ -491,7 +598,10 @@ class UserModuleDeleter(object):
     def __init__(self, namelist=None, pathlist=None):
         if namelist is None:
             namelist = []
-        self.namelist = namelist+['sitecustomize', 'spyderlib', 'spyderplugins']
+        spy_modules = ['sitecustomize', 'spyderlib', 'spyderplugins']
+        mpl_modules = ['matplotlib', 'tkinter', 'Tkinter', 'gtk']
+        self.namelist = namelist + spy_modules + mpl_modules
+        
         if pathlist is None:
             pathlist = []
         self.pathlist = pathlist
@@ -526,11 +636,14 @@ class UserModuleDeleter(object):
                     del sys.modules[modname]
         if verbose and log:
             _print("\x1b[4;33m%s\x1b[24m%s\x1b[0m"\
-                   % ("UMD has deleted", ": "+", ".join(log)))
+                   % ("Reloaded modules", ": "+", ".join(log)))
 
-__umd__ = None
+__umr__ = None
 
 
+#==============================================================================
+# runfile and debugfile commands
+#==============================================================================
 def _get_globals():
     """Return current Python interpreter globals namespace"""
     from __main__ import __dict__ as namespace
@@ -556,16 +669,16 @@ def runfile(filename, args=None, wdir=None, namespace=None):
         # UnicodeError, TypeError --> eventually raised in Python 2
         # AttributeError --> systematically raised in Python 3
         pass
-    global __umd__
-    if os.environ.get("UMD_ENABLED", "").lower() == "true":
-        if __umd__ is None:
-            namelist = os.environ.get("UMD_NAMELIST", None)
+    global __umr__
+    if os.environ.get("UMR_ENABLED", "").lower() == "true":
+        if __umr__ is None:
+            namelist = os.environ.get("UMR_NAMELIST", None)
             if namelist is not None:
                 namelist = namelist.split(',')
-            __umd__ = UserModuleDeleter(namelist=namelist)
+            __umr__ = UserModuleReloader(namelist=namelist)
         else:
-            verbose = os.environ.get("UMD_VERBOSE", "").lower() == "true"
-            __umd__.run(verbose=verbose)
+            verbose = os.environ.get("UMR_VERBOSE", "").lower() == "true"
+            __umr__.run(verbose=verbose)
     if args is not None and not isinstance(args, basestring):
         raise TypeError("expected a character buffer object")
     if namespace is None:
@@ -608,55 +721,55 @@ def debugfile(filename, args=None, wdir=None):
 builtins.debugfile = debugfile
 
 
+#==============================================================================
+# Evaluate external commands
+#==============================================================================
 def evalsc(command):
     """Evaluate special commands
     (analog to IPython's magic commands but far less powerful/complete)"""
-    assert command.startswith(('%', '!'))
-    system_command = command.startswith('!')
-    command = command[1:].strip()
-    if system_command:
-        # System command
-        if command.startswith('cd '):
-            evalsc('%'+command)
-        else:
-            from subprocess import Popen, PIPE
-            Popen(command, shell=True, stdin=PIPE)
-            _print('\n')
-    else:
-        # General command
-        namespace = _get_globals()
-        import re
-        clear_match = re.match(r"^clear ([a-zA-Z0-9_, ]+)", command)
-        cd_match = re.match(r"^cd \"?\'?([a-zA-Z0-9_\ \:\\\/\.]+)", command)
-        if cd_match:
-            os.chdir(eval('r"%s"' % cd_match.groups()[0].strip()))
-        elif clear_match:
-            varnames = clear_match.groups()[0].replace(' ', '').split(',')
-            for varname in varnames:
-                try:
-                    namespace.pop(varname)
-                except KeyError:
-                    pass
-        elif command in ('cd', 'pwd'):
+    assert command.startswith('%')
+    
+    from subprocess import Popen, PIPE
+    namespace = _get_globals()
+    command = command[1:].strip()  # Remove leading %
+
+    import re
+    clear_match = re.match(r"^clear ([a-zA-Z0-9_, ]+)", command)
+    cd_match = re.match(r"^cd \"?\'?([a-zA-Z0-9_\ \:\\\/\.]+)", command)
+
+    if cd_match:
+        os.chdir(eval('r"%s"' % cd_match.groups()[0].strip()))
+    elif clear_match:
+        varnames = clear_match.groups()[0].replace(' ', '').split(',')
+        for varname in varnames:
             try:
-                _print(os.getcwdu())
-            except AttributeError:
-                _print(os.getcwd())
-        elif command == 'ls':
-            if os.name == 'nt':
-                evalsc('!dir')
-            else:
-                evalsc('!ls')
-        elif command == 'scientific':
-            from spyderlib import baseconfig
-            execfile(baseconfig.SCIENTIFIC_STARTUP, namespace)
+                namespace.pop(varname)
+            except KeyError:
+                pass
+    elif command in ('cd', 'pwd'):
+        try:
+            _print(os.getcwdu())
+        except AttributeError:
+            _print(os.getcwd())
+    elif command == 'ls':
+        if os.name == 'nt':
+            Popen('dir', shell=True, stdin=PIPE)
+            _print('\n')
         else:
-            raise NotImplementedError("Unsupported command: '%s'" % command)
+            Popen('ls', shell=True, stdin=PIPE)
+            _print('\n')
+    elif command == 'scientific':
+        from spyderlib import baseconfig
+        execfile(baseconfig.SCIENTIFIC_STARTUP, namespace)
+    else:
+        raise NotImplementedError("Unsupported command: '%s'" % command)
 
 builtins.evalsc = evalsc
 
 
+#==============================================================================
 # Restoring original PYTHONPATH
+#==============================================================================
 try:
     os.environ['PYTHONPATH'] = os.environ['OLD_PYTHONPATH']
     del os.environ['OLD_PYTHONPATH']
