@@ -10,32 +10,30 @@
 import sys
 import os.path as osp
 
+# TODO: Move to Jupyter imports in 3.1
+try:
+    import warnings
+    from IPython.utils.shimmodule import ShimWarning
+    warnings.simplefilter('ignore', ShimWarning)
+except:
+    pass
 
-def sympy_config():
+
+def sympy_config(mpl_backend):
     """Sympy configuration"""
-    from spyderlib.utils.programs import is_module_installed    
-    lines_new = """
+    if mpl_backend is not None:
+        lines = """
 from sympy.interactive import init_session
 init_session()
-"""    
-    lines_old = """
-from __future__ import division
-from sympy import *
-x, y, z, t = symbols('x y z t')
-k, m, n = symbols('k m n', integer=True)
-f, g, h = symbols('f g h', cls=Function)
-"""
-    if is_module_installed('sympy', '>=0.7.3'):
-        extension = None
-        return lines_new, extension
-    elif is_module_installed('sympy', '=0.7.2'):
-        extension = 'sympy.interactive.ipythonprinting'
-        return lines_old, extension
-    elif is_module_installed('sympy', '>=0.7.0;<0.7.2'):
-        extension = 'sympyprinting'
-        return lines_old, extension
+%matplotlib {0}
+""".format(mpl_backend)
     else:
-        return None, None
+        lines = """
+from sympy.interactive import init_session
+init_session()
+"""
+
+    return lines
 
 
 def kernel_config():
@@ -57,24 +55,38 @@ def kernel_config():
     # ---- Spyder config ----
     spy_cfg = Config()
     
-    # Until we implement Issue 1052:
-    # http://code.google.com/p/spyderlib/issues/detail?id=1052
+    # Until we implement Issue 1052
     spy_cfg.InteractiveShell.xmode = 'Plain'
     
+    # Run lines of code at startup
+    run_lines_o = CONF.get('ipython_console', 'startup/run_lines')
+    if run_lines_o:
+        spy_cfg.IPKernelApp.exec_lines = [x.strip() for x in run_lines_o.split(',')]
+    else:
+        spy_cfg.IPKernelApp.exec_lines = []
+    
     # Pylab configuration
+    mpl_backend = None
     mpl_installed = is_module_installed('matplotlib')
     pylab_o = CONF.get('ipython_console', 'pylab')
-    
+
     if mpl_installed and pylab_o:
+        # Get matplotlib backend
         backend_o = CONF.get('ipython_console', 'pylab/backend', 0)
         backends = {0: 'inline', 1: 'auto', 2: 'qt', 3: 'osx', 4: 'gtk',
                     5: 'wx', 6: 'tk'}
-        spy_cfg.IPKernelApp.pylab = backends[backend_o]
-        
-        # Automatically load Pylab and Numpy
+        mpl_backend = backends[backend_o]
+
+        # Automatically load Pylab and Numpy, or only set Matplotlib
+        # backend
         autoload_pylab_o = CONF.get('ipython_console', 'pylab/autoload')
-        spy_cfg.IPKernelApp.pylab_import_all = autoload_pylab_o
-        
+        if autoload_pylab_o:
+            spy_cfg.IPKernelApp.exec_lines.append(
+                                              "%pylab {0}".format(mpl_backend))
+        else:
+            spy_cfg.IPKernelApp.exec_lines.append(
+                                         "%matplotlib {0}".format(mpl_backend))
+
         # Inline backend configuration
         if backends[backend_o] == 'inline':
            # Figure format
@@ -100,12 +112,6 @@ def kernel_config():
            height_o = float(CONF.get('ipython_console', 'pylab/inline/height'))
            spy_cfg.InlineBackend.rc['figure.figsize'] = (width_o, height_o)
     
-    # Run lines of code at startup
-    run_lines_o = CONF.get('ipython_console', 'startup/run_lines')
-    if run_lines_o:
-        spy_cfg.IPKernelApp.exec_lines = map(lambda x: x.strip(),
-                                         run_lines_o.split(','))
-    
     # Run a file at startup
     use_file_o = CONF.get('ipython_console', 'startup/use_run_file')
     run_file_o = CONF.get('ipython_console', 'startup/run_file')
@@ -116,6 +122,9 @@ def kernel_config():
     autocall_o = CONF.get('ipython_console', 'autocall')
     spy_cfg.ZMQInteractiveShell.autocall = autocall_o
     
+    # To handle the banner by ourselves in IPython 3+
+    spy_cfg.ZMQInteractiveShell.banner1 = ''
+    
     # Greedy completer
     greedy_o = CONF.get('ipython_console', 'greedy_completer')
     spy_cfg.IPCompleter.greedy = greedy_o
@@ -123,16 +132,9 @@ def kernel_config():
     # Sympy loading
     sympy_o = CONF.get('ipython_console', 'symbolic_math')
     if sympy_o:
-        lines, extension = sympy_config()
-        if lines is not None:
-            if run_lines_o:
-                spy_cfg.IPKernelApp.exec_lines.append(lines)
-            else:
-                spy_cfg.IPKernelApp.exec_lines = [lines]
-            if extension:
-                spy_cfg.IPKernelApp.extra_extension = extension
-                spy_cfg.LaTeXTool.backends = ['dvipng', 'matplotlib']
-    
+        lines = sympy_config(mpl_backend)
+        spy_cfg.IPKernelApp.exec_lines.append(lines)
+
     # Merge IPython and Spyder configs. Spyder prefs will have prevalence
     # over IPython ones
     ip_cfg._merge(spy_cfg)
@@ -148,6 +150,20 @@ def change_edit_magic(shell):
     except:
         pass
 
+def varexp(line):
+    """
+    Spyder's variable explorer magic
+    
+    Used to generate plots, histograms and images of the variables displayed
+    on it.
+    """
+    ip = get_ipython()       #analysis:ignore
+    funcname, name = line.split()
+    import spyderlib.pyplot
+    __fig__ = spyderlib.pyplot.figure();
+    __items__ = getattr(spyderlib.pyplot, funcname[2:])(ip.user_ns[name])
+    spyderlib.pyplot.show()
+    del __fig__, __items__
 
 # Remove this module's path from sys.path:
 try:
@@ -164,11 +180,7 @@ __name__ = '__main__'
 sys.path.insert(0, '')
 
 # Fire up the kernel instance.
-try:
-    from IPython.kernel.zmq.kernelapp import IPKernelApp  # 1.0
-except ImportError:
-    from IPython.zmq.ipkernel import IPKernelApp  # 0.13  (analysis:ignore)
-
+from IPython.kernel.zmq.kernelapp import IPKernelApp
 ipk_temp = IPKernelApp.instance()
 ipk_temp.config = kernel_config()
 ipk_temp.initialize()
@@ -187,11 +199,7 @@ del ipk_temp
 # NOTE: Leave this and other magic modifications *after* setting
 # __ipythonkernel__ to not have problems while starting kernels
 change_edit_magic(__ipythonshell__)
-
-# To make %pylab load numpy and pylab even if the user has
-# set autoload_pylab_o to False *but* nevertheless use it in
-# the interactive session.
-__ipythonkernel__.pylab_import_all = True
+__ipythonshell__.register_magic_function(varexp)
 
 # Start the (infinite) kernel event loop.
 __ipythonkernel__.start()
