@@ -33,15 +33,11 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 """
 
-# ----+- Python 3 compatibility -+----
 from __future__ import print_function
-try:
-    basestring
-except NameError:
-    # Python 3
-    basestring = unicode = str
 
 # History:
+# 1.0.15: added support for multiline strings
+# 1.0.14: fixed Python 3 support (regression in 1.0.13)
 # 1.0.13: replaced obsolete QColorDialog.getRgba function and fixed other 
 #         compatibility issues with PySide (see Issue 8 of formlayout website)
 # 1.0.12: added support for Python 3
@@ -50,9 +46,10 @@ except NameError:
 # 1.0.7: added support for "Apply" button
 # 1.0.6: code cleaning
 
-__version__ = '1.0.13'
+__version__ = '1.0.15'
 __license__ = __doc__
 
+import os
 
 try:
     from spyderlib.qt.QtGui import QFormLayout
@@ -65,12 +62,14 @@ from spyderlib.qt.QtGui import (QWidget, QLineEdit, QComboBox, QLabel,
                                 QPushButton, QCheckBox, QColorDialog, QPixmap,
                                 QTabWidget, QApplication, QStackedWidget,
                                 QDateEdit, QDateTimeEdit, QFont, QFontComboBox,
-                                QFontDatabase, QGridLayout, QDoubleValidator)
+                                QFontDatabase, QGridLayout, QDoubleValidator,
+                                QTextEdit)
 from spyderlib.qt.QtCore import Qt, SIGNAL, SLOT, QSize, Slot, Property
 import datetime
 
 # Local imports
 from spyderlib.baseconfig import _, DEBUG, STDERR
+from spyderlib.py3compat import is_text_string, to_text_string, is_string, u
 
 DEBUG_FORMLAYOUT = DEBUG >= 2
 
@@ -114,9 +113,9 @@ def text_to_qcolor(text):
     Avoid warning from Qt when an invalid QColor is instantiated
     """
     color = QColor()
-    if not isinstance(text, basestring): # testing for QString (PyQt API#1)
+    if not is_string(text): # testing for QString (PyQt API#1)
         text = str(text)
-    if not isinstance(text, (unicode, str)):
+    if not is_text_string(text):
         return color
     if text.startswith('#') and len(text)==7:
         correct = '#0123456789abcdef'
@@ -158,7 +157,8 @@ class ColorLayout(QHBoxLayout):
     
 def font_is_installed(font):
     """Check if font is installed"""
-    return [fam for fam in QFontDatabase().families() if unicode(fam)==font]
+    return [fam for fam in QFontDatabase().families()
+            if to_text_string(fam) == font]
 
 def tuple_to_qfont(tup):
     """
@@ -180,7 +180,7 @@ def tuple_to_qfont(tup):
     return font
 
 def qfont_to_tuple(font):
-    return (unicode(font.family()), int(font.pointSize()),
+    return (to_text_string(font.family()), int(font.pointSize()),
             font.italic(), font.bold())
 
 class FontLayout(QGridLayout):
@@ -272,8 +272,14 @@ class FormWidget(QWidget):
                 field = FontLayout(value, self)
             elif text_to_qcolor(value).isValid():
                 field = ColorLayout(QColor(value), self)
-            elif isinstance(value, (str, unicode)):
-                field = QLineEdit(value, self)
+            elif is_text_string(value):
+                if '\n' in value:
+                    for linesep in (os.linesep, '\n'):
+                        if linesep in value:
+                            value = value.replace(linesep, u("\u2029"))
+                    field = QTextEdit(value, self)
+                else:
+                    field = QLineEdit(value, self)
             elif isinstance(value, (list, tuple)):
                 value = list(value)  # in case this is a tuple
                 selindex = value.pop(0)
@@ -328,8 +334,12 @@ class FormWidget(QWidget):
                 continue
             elif tuple_to_qfont(value) is not None:
                 value = field.get_font()
-            elif isinstance(value, (str, unicode)):
-                value = unicode(field.text())
+            elif is_text_string(value):
+                if isinstance(field, QTextEdit):
+                    value = to_text_string(field.toPlainText()
+                                           ).replace(u("\u2029"), os.linesep)
+                else:
+                    value = to_text_string(field.text())
             elif isinstance(value, (list, tuple)):
                 index = int(field.currentIndex())
                 if isinstance(value[0], int):
@@ -532,6 +542,9 @@ if __name__ == "__main__":
 
     def create_datalist_example():
         return [('str', 'this is a string'),
+                ('str', """this is a 
+                MULTILINE
+                string"""),
                 ('list', [0, '1', '3', '4']),
                 ('list2', ['--', ('none', 'None'), ('--', 'Dashed'),
                            ('-.', 'DashDot'), ('-', 'Solid'),
