@@ -8,6 +8,8 @@
 Text Editor Dialog based on Qt
 """
 
+from __future__ import print_function
+
 from spyderlib.qt.QtCore import Qt, SIGNAL, SLOT
 from spyderlib.qt.QtGui import QVBoxLayout, QTextEdit, QDialog, QDialogButtonBox
 
@@ -15,6 +17,8 @@ from spyderlib.qt.QtGui import QVBoxLayout, QTextEdit, QDialog, QDialogButtonBox
 from spyderlib.baseconfig import _
 from spyderlib.guiconfig import get_font
 from spyderlib.utils.qthelpers import get_icon
+from spyderlib.py3compat import (to_text_string, to_binary_string,
+                                 is_binary_string)
 
 
 class TextEditor(QDialog):
@@ -31,7 +35,13 @@ class TextEditor(QDialog):
         
         self.text = None
         
-        self._conv = str if isinstance(text, str) else unicode
+        # Display text as unicode if it comes as bytes, so users see 
+        # its right representation
+        if is_binary_string(text):
+            self.is_binary = True
+            text = to_text_string(text, 'utf8')
+        else:
+            self.is_binary = False
         
         self.layout = QVBoxLayout()
         self.setLayout(self.layout)
@@ -65,7 +75,11 @@ class TextEditor(QDialog):
     
     def text_changed(self):
         """Text has changed"""
-        self.text = self._conv(self.edit.toPlainText())
+        # Save text as bytes, if it was initially bytes
+        if self.is_binary:
+            self.text = to_binary_string(self.edit.toPlainText(), 'utf8')
+        else:
+            self.text = to_text_string(self.edit.toPlainText())
         
     def get_value(self):
         """Return modified text"""
@@ -85,11 +99,11 @@ def test():
     dialog.show()
     if dialog.exec_():
         text = dialog.get_value()
-        print "Accepted:", text
+        print("Accepted:", text)
         dialog = TextEditor(text)
         dialog.exec_()
     else:
-        print "Canceled"
+        print("Canceled")
 
 if __name__ == "__main__":
     test()
\ No newline at end of file
