@@ -73,11 +73,10 @@ class ProfilerWidget(QWidget):
                                     tip=_("Run profiler"),
                                     triggered=self.start, text_beside_icon=True)
         self.stop_button = create_toolbutton(self,
-                                    icon=get_icon('terminate.png'),
-                                    text=_("Stop"),
-                                    tip=_(
-                                                  "Stop current profiling"),
-                                    text_beside_icon=True)
+                                             icon=get_icon('stop.png'),
+                                             text=_("Stop"),
+                                             tip=_("Stop current profiling"),
+                                             text_beside_icon=True)
         self.connect(self.filecombo, SIGNAL('valid(bool)'),
                      self.start_button.setEnabled)
         #self.connect(self.filecombo, SIGNAL('valid(bool)'), self.show_data)
@@ -299,6 +298,18 @@ class ProfilerWidget(QWidget):
         self.datelabel.setText(date_text)
 
 
+class TreeWidgetItem( QTreeWidgetItem ):
+    def __init__(self, parent=None):
+        QTreeWidgetItem.__init__(self, parent)
+    
+    def __lt__(self, otherItem):
+        column = self.treeWidget().sortColumn()
+        try:
+            return float( self.text(column) ) > float( otherItem.text(column) )
+        except ValueError:
+            return self.text(column) > otherItem.text(column)
+
+
 class ProfilerDataTree(QTreeWidget):
     """
     Convenience tree widget (with built-in model) 
@@ -369,7 +380,9 @@ class ProfilerDataTree(QTreeWidget):
         """Find a function without a caller"""
         self.profdata.sort_stats("cumulative")
         for func in self.profdata.fcn_list:
-            if ('~', 0, '<built-in method exec>') != func:
+            if ('~', 0, '<built-in method exec>') != func: 
+                # This skips the profiler function at the top of the list
+                # it does only occur in Python 3
                 return func
     
     def find_callees(self, parent):
@@ -418,7 +431,7 @@ class ProfilerDataTree(QTreeWidget):
              ) = self.function_info(child_key)
             (primcalls, total_calls, loc_time, cum_time, callers
              ) = self.stats[child_key]
-            child_item = QTreeWidgetItem(parentItem)
+            child_item = TreeWidgetItem(parentItem)
             self.item_list.append(child_item)
             self.set_item_data(child_item, filename, line_number)
 
